@@ -3,7 +3,7 @@ package Plack::Middleware::XSRFBlock;
 {
   $Plack::Middleware::XSRFBlock::DIST = 'Plack-Middleware-XSRFBlock';
 }
-$Plack::Middleware::XSRFBlock::VERSION = '0.0.13';
+$Plack::Middleware::XSRFBlock::VERSION = '0.0.14';
 use strict;
 use warnings;
 use parent 'Plack::Middleware';
@@ -21,6 +21,8 @@ use Plack::Util::Accessor qw(
     cookie_name
     cookie_is_session_cookie
     cookie_options
+    http_method_regex
+    contents_to_filter_regex
     inject_form_input
     logger
     meta_tag
@@ -40,6 +42,15 @@ sub prepare_app {
     # default to 1 so we inject hidden inputs to forms
     $self->inject_form_input(1) unless defined $self->inject_form_input;
 
+    # match methods
+    $self->http_method_regex( $self->http_method_regex || qr{^post$}i );
+
+    # match content types
+    $self->contents_to_filter_regex(
+        $self->contents_to_filter_regex ||
+            qr{^(?: (?:text/html) | (?:application/xhtml(?:\+xml)?) )$}ix,
+    );
+
     # store the cookie_name
     $self->cookie_name( $self->cookie_name || 'PSGI-XSRF-Token' );
 
@@ -74,6 +85,7 @@ sub prepare_app {
     });
 }
 
+
 sub detect_xsrf {
     my $self    = shift;
     my $request = shift;
@@ -131,8 +143,8 @@ sub call {
     my $request = Plack::Request->new($env);
 
     # deal with form posts
-    if ($request->method =~ m{^post$}i) {
-        $self->log(info => 'POST submitted');
+    if ($request->method =~ $self->http_method_regex) {
+        $self->log(info => 'form submitted');
 
         my $msg = $self->detect_xsrf($request, $env);
         return $self->xsrf_detected({ env => $env, msg => $msg })
@@ -142,30 +154,29 @@ sub call {
     return $self->filter_response($request, $env);
 }
 
-sub cookie_handler {
+
+sub should_be_filtered {
     my ($self, $request, $env, $res) = @_;
 
-    # grab the cookie where we store the token
-    my $cookie_value = $request->cookies->{$self->cookie_name};
-
-    # Determine whether to create a new token, based on the request
-    $cookie_value = $self->_token_generator->()
-        if $self->token_per_request->( $self, $request, $env );
-
-    # make it easier to work with the headers
     my $headers = Plack::Util::headers($res->[1]);
-
-    # we can't form-munge anything non-HTML
     my $ct = $headers->get('Content-Type') || '';
-    if($ct !~ m{^text/html}i and $ct !~ m{^application/xhtml[+]xml}i){
-        return $res;
-    }
+    return !! ($ct =~ $self->contents_to_filter_regex);
+}
 
-    # GITHUB ISSUE #12 - set cookie after we're happy it's HTML
-    # get the token value from:
-    # - cookie value, if it's already set
-    # - from the generator, if we don't have one yet
-    my $token = $cookie_value ||= $self->_token_generator->();
+
+sub generate_token {
+    my ($self, $request, $env, $res) = @_;
+
+    my $token = $request->cookies->{$self->cookie_name};
+    $token = $self->_token_generator->()
+        if !$token or $self->token_per_request->( $self, $request, $env );
+
+    return $token;
+}
+
+
+sub cookie_handler {
+    my ($self, $request, $env, $res, $token) = @_;
 
     my %cookie_expires;
     unless ( $self->cookie_is_session_cookie ) {
@@ -180,9 +191,10 @@ sub cookie_handler {
         %cookie_expires,
     );
 
-    return $token;
+    return;
 }
 
+
 sub filter_response_html {
     my ($self, $request, $env, $res, $token) = @_;
 
@@ -232,12 +244,12 @@ sub filter_response_html {
                     );
             }
 
-            # If tag isn't 'form' and method isn't 'post' we dont care
+            # If tag isn't 'form' and method isn't matched, we dont care
             return unless
                     defined $tag
                 && defined $attr->{'method'}
                 && $tag eq 'form'
-                && $attr->{'method'} =~ /post/i;
+                && $attr->{'method'} =~ $self->http_method_regex;
 
             if(
                 !(
@@ -290,13 +302,18 @@ sub filter_response_html {
     }
 }
 
+
 sub filter_response {
     my ($self, $request, $env) = @_;
 
     return Plack::Util::response_cb($self->app->($env), sub {
         my $res = shift;
 
-        my $token = $self->cookie_handler($request, $env, $res);
+        return $res unless $self->should_be_filtered($request, $env, $res);
+
+        my $token = $self->generate_token($request, $env, $res);
+
+        $self->cookie_handler($request, $env, $res, $token);
 
         return $res unless $self->inject_form_input;
 
@@ -304,11 +321,12 @@ sub filter_response {
     });
 }
 
+
 sub invalid_signature {
     my ($self, $value) = @_;
 
     # we dont use signed cookies
-    return if !defined $self->secret;
+    return 0 if !defined $self->secret;
 
     # cookie isn't signed
     my ($token, $signature) = split /--/, $value;
@@ -318,6 +336,7 @@ sub invalid_signature {
     return hmac_sha1_hex($token, $self->secret) ne $signature;
 }
 
+
 sub xsrf_detected {
     my $self    = shift;
     my $args    = shift;
@@ -339,6 +358,7 @@ sub xsrf_detected {
     ];
 }
 
+
 sub log {
     my ($self, $level, $msg) = @_;
     $self->logger->({ level => $level, message => "XSRFBlock: $msg" });
@@ -378,7 +398,7 @@ Plack::Middleware::XSRFBlock - Block XSRF Attacks with minimal changes to your a
 
 =head1 VERSION
 
-version 0.0.13
+version 0.0.14
 
 =head1 SYNOPSIS
 
@@ -406,6 +426,8 @@ You may also over-ride any, or all of these values:
             inject_form_input       => 1,
             header_name             => undef,
             secret                  => undef,
+            http_method_regex       => qr{^post$}i,
+            contents_to_filter_regex => qr{^(text/html|application/xhtml(?:\+xml)?)$}i,
             blocked                 => sub {
                                         return [ $status, $headers, $body ]
                                     },
@@ -458,6 +480,15 @@ double-submit issues.
 
 If this is a coderef, the coderef will be evaluated with the following arguments:
 
+=item http_method_regex (default: qr{^post$}i)
+
+Which HTTP methods to check. Can be useful to also handle PUT, DELETE,
+PATCH, and the like.
+
+=item contents_to_filter_regex default: qr{^(text/html|application/xhtml(?:\+xml)?)$}i)
+
+Only modify <form> elements in responses whose content type matches this regex
+
 =over
 
 =item * The middleware object itself,
@@ -545,18 +576,83 @@ to the browser.
 
 =head2 detect_xsrf($self, $request, $env)
 
-=head2 cookie_handler($self, $request, $env, $res)
+returns a message explaining the XSRF-related problem, or C<undef> if
+there's no problem
+
+=head2 should_be_filtered($self, $request, $env, $res)
+
+returns true if the response should be filtered by this middleware
+(currently, if its content-type matches C<contents_to_filter_regex>)
+
+=head2 generate_token($self, $request, $env, $res)
+
+returns the token value to use for this response.  Gets the token value from:
+
+=over
+
+=item *
+
+the cookie value, if it's already set
+
+=item *
+
+from the generator, if the cookie is not set, or if we want a
+different token for each request
+
+=back
+
+=head2 cookie_handler($self, $request, $env, $res, $token)
+
+sets the given token as a cookie in the response
 
 =head2 filter_response_html($self, $request, $env, $res, $token)
 
+Filters the response, injecting C<< <input> >> elements with the token
+value into all forms whose method matches C<http_method_regex>.
+
+Streaming responses are still streaming after the filtering.
+
 =head2 filter_response($self, $request, $env)
 
+Calls the application, and (if the response L<< /C<should_be_filtered>
+>>), it injects the token in the cookie and (if L<<
+/C<inject_form_input> >>) the forms.
+
 =head2 invalid_signature($self, $value)
 
+Returns true if the value is not correctly signed. If we're not
+signing tokens, this method always returns false.
+
 =head2 xsrf_detected($self, $args)
 
+Invoked when the XSRF is detected. Calls the L<< /C<blocked> >>
+coderef if we have it, or returns a 403.
+
+The C<blocked> coderef is invoked like:
+
+  $self->blocked->($env,$msg, app => $self->app);
+
+=over
+
+=item *
+
+the original request PSGI environment
+
+=item *
+
+the error message (from L<< /C<detect_xsrf> >>)
+
+=item *
+
+a hash, currently C<< app => $self->app >>, so you can call the
+original application
+
+=back
+
 =head2 log($self, $level, $msg)
 
+log through the PSGI logger, if defined
+
 =head1 EXPLANATION
 
 This module is similar in nature and intention to
@@ -647,7 +743,7 @@ the same terms as the Perl 5 programming language system itself.
 
 =head1 CONTRIBUTORS
 
-=for stopwords Chisel Daniel Perrett Karen Etheridge Matthew Ryall Matthias Zeichmann Michael Kröll Sebastian Willert Sterling Hanenkamp William Wolf
+=for stopwords Chisel Daniel Perrett Gianni Ceccarelli Karen Etheridge Matthew Ryall Matthias Zeichmann Michael Kröll Sebastian Willert Sterling Hanenkamp William Wolf
 
 =over 4
 
@@ -661,6 +757,10 @@ Daniel Perrett <dp13@sanger.ac.uk>
 
 =item *
 
+Gianni Ceccarelli <gianni.ceccarelli@broadbean.com>
+
+=item *
+
 Karen Etheridge <ether@cpan.org>
 
 =item *
